package eu.mhsl.craftattack.spawn.appliances.report;

import com.google.gson.Gson;
import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.appliances.report.command.ReportCommand;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.event.ClickEvent;
import net.kyori.adventure.text.event.HoverEvent;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.bukkit.OfflinePlayer;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;

import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.List;
import java.util.UUID;

public class Report extends Appliance {
    private record Request(UUID reporter, UUID reported, String reason) {}
    private record Response(String url) {}

    public void issueReport(Player issuer, String targetUsername, String reason) {
        try {
            OfflinePlayer target = Bukkit.getOfflinePlayer(targetUsername);
            Request request = new Request(issuer.getUniqueId(), target.getUniqueId(), reason);

            HttpResponse<String> responseString = postRequest(URI.create("https://mhsl.eu/craftattack/report"), new Gson().toJson(request));

            switch (responseString.statusCode()) {
                case 201:
                    Response response = new Gson().fromJson(responseString.body(), Response.class);
                    issuer.sendMessage(
                            Component.text()
                                    .append(Component.text("\\/".repeat(10), NamedTextColor.DARK_GRAY))
                                    .appendNewline()
                                    .appendNewline()
                                    .append(Component.text("!!! Report hier fertigstellen !!!", NamedTextColor.GOLD))
                                    .appendNewline()
                                    .appendNewline()
                                    .append(
                                            Component
                                                    .text(" > Hier klicken < ", NamedTextColor.GREEN)
                                                    .hoverEvent(HoverEvent.showText(Component.text(response.url).color(NamedTextColor.GREEN)))
                                                    .clickEvent(ClickEvent.clickEvent(ClickEvent.Action.OPEN_URL, response.url))
                                    )
                                    .appendNewline()
                                    .appendNewline()
                                    .append(
                                            Component
                                                    .text("(" + response.url + ")", NamedTextColor.GRAY)
                                                    .clickEvent(ClickEvent.clickEvent(ClickEvent.Action.OPEN_URL, response.url))
                                    )
                                    .appendNewline()
                                    .appendNewline()
                                    .append(Component.text("/\\".repeat(10), NamedTextColor.DARK_GRAY))
                    );
                    break;

                case 400:
                    issuer.sendMessage(
                            Component.text()
                                    .append(Component.text("Der Nutzer ", NamedTextColor.RED))
                                    .append(Component.text(targetUsername, NamedTextColor.DARK_RED))
                                    .append(Component.text(" ist in unserem System nicht bekannt.", NamedTextColor.RED))
                                    .appendNewline()
                                    .append(Component.text("Bist du sicher, dass du den Namen richtig geschrieben hast?", NamedTextColor.RED))
                                    .appendNewline()
                                    .append(Component.text("Du kannst dich alternativ jederzeit bei einem Admin melden.", NamedTextColor.GRAY))
                    );
                    break;

                case 401:
                default:
                    issuer.sendMessage(
                            Component.text()
                                    .append(Component.text("Interner Serverfehler beim anlegen des Reports.", NamedTextColor.RED))
                                    .appendNewline()
                                    .append(Component.text("Bitte melde dich bei einem Admin!", NamedTextColor.RED))
                    );
                    break;
            }



        } catch (IOException | InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

    private HttpResponse<String> postRequest(URI uri, String body) throws IOException, InterruptedException {
        HttpClient client = HttpClient.newHttpClient();
        HttpRequest httpRequest = HttpRequest.newBuilder()
                .uri(uri)
                .header("Content-Type", "application/json")
                .POST(HttpRequest.BodyPublishers.ofString(body))
                .build();

        return client.send(httpRequest, HttpResponse.BodyHandlers.ofString());
    }

    @Override
    protected @NotNull List<ApplianceCommand<?>> commands() {
        return List.of(new ReportCommand());
    }
}
